<?php

namespace EventPortal\Domain\Util\Menu;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;

class Menu
{
    /** @var \Illuminate\Support\Collection */
    protected $actions = [];

    /** @var Model */
    protected $model;

    /** @var Authenticatable */
    protected $user;

    public function user(Authenticatable $user): self
    {
        $this->user = $user;

        return $this;
    }

    public function model(Model $model): self
    {
        $this->model = $model;

        return $this;
    }

    public function addLinks(callable $callable): self
    {
        $this->actions[] = $callable;

        return $this;
    }

    public function render(): array
    {
        $this->resolveLinks()
            ->filterPermittedLinks()
            ->renderLinks();

        return [
            'children' => array_values($this->actions->toArray()),
        ];
    }

    private function resolveLinks(): self
    {
        $this->actions = collect($this->actions)
            ->transform(function ($action) {
                return $action($this->model);
            })
            ->flatten(1); // addLinks can be called multiple times

        return $this;
    }

    private function filterPermittedLinks(): self
    {
        $this->actions = $this
            ->actions
            ->filter(function (Link $link) {
                return $link->canSee;
            })
            ->map(function (Link $link) {
                if ($link->children === null) {
                    return $link;
                }


                $link->children = array_filter($link->children, function ($link) {
                    return $link->canSee;
                });

                $link->children = array_values($link->children);

                return $link;
            });

        return $this;
    }

    private function renderLinks(): self
    {
        $this->actions = $this->actions->transform(function (Link $link) {
            return $link->render();
        });

        return $this;
    }
}
